/*
** Module   : WidgetFactory.java
** Abstract :
**
** Copyright (c) 2011-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20111221          Created initial version
** 002 MAG 20140410          Add RECTANGLE widget creation.
** 003 EVL 20140610          Added TOGGLE-BOX widget support.
*/

package com.goldencode.p2j.ui.client.driver;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.model.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * This interface provides access to driver-specific widget factory necessary to 
 * hide from client-level code implementations of driver-specific widgets.
 */
public interface WidgetFactory
{
   /**
    * Create new {@link Label} instance for given text.
    * 
    * @param   text
    *          Label text. 
    * @return  new {@link Label} instance.
    */
   Label createLabel(String text);

   /**
    * Create new button for given container and text.
    * 
    * @param   container
    *          Container to which new instance will belong.
    * @param   text
    *          Button text.
    *          
    * @return  new {@link Button} instance.
    */
   Button createButton(Container container, String text);

   /**
    * Create new button for given container and text.
    * 
    * @param   container
    *          Container to which new instance will belong.
    * @param   text
    *          ToggleBox label text.
    *          
    * @return  new {@link Button} instance.
    */
   ToggleBox createToggleBox(Container container, String text);

   /**
    * Create new rectangle for given container.
    * 
    * @param   container
    *          Container to which new instance will belong.
    *          
    * @return  new {@link Rect} instance.
    */
   Rect createRectangle(Container container);

   /**
    * Instantiate UI-specific version of the specified widget class.
    * 
    * @param   <T>
    *          Widget abstract class type.
    * @param   clazz
    *          Widget abstract class for which UI-specific implementation should
    *          be instantiated.
    * @param   cfg
    *          Widget configuration container.
    *          
    * @return  new UI-specific instance of the requested class. 
    */
   <T extends Widget> T create(Class<T> clazz, ComponentConfig cfg);

   /**
    * Create UI-specific {@link ScrollableList} implementation.
    * 
    * @param   parent
    *          Parent {@link ComboBox} widget.
    * @param   model
    *          Scrollable list model.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollableList}.
    */
   ScrollableList createScrollableList(ComboBox parent,
                                       ListModel<String> model);

   /**
    * Create UI-specific {@link ScrollPane} implementation.
    * 
    * @param   widget 
    *          Widget which will be scrolled.
    *          
    * @return  instance of UI-specific implementation of {@link ScrollPane}.
    */
   ScrollPane createScrollPane(Widget widget);

   /**
    * Create UI-specific {@link Label} instance as a copy of given {@link Label}.
    * 
    * @param   labelInstance
    *          Source {@link Label} instance.
    *          
    * @return  instance of UI-specific implementation of {@link Label}.
    */
   Label createLabel(Label labelInstance);
   
   /**
    * Create UI-specific {@link RadioButton} instance.
    * @param   config
    *          Parent config. 
    * @param   label
    *          Radio button label. 
    * @param   value
    *          Radio button data type. 
    * 
    * @return  instance of UI-specific implementation of {@link RadioButton}.
    */
   RadioButton createRadioButton(RadioSetConfig config, String label,
                                 BaseDataType value);

   /**
    * Create UI-specific {@link Window} instance.
    * 
    * @return  instance of UI-specific implementation of {@link Window}.
    */
   Window createWindow();

   /**
    * Create UI-specific {@link MessageLine} instance.
    * 
    * @return  instance of UI-specific implementation of {@link MessageLine}.
    */
   StatusLine createMessageLine();

   /**
    * Create UI-specific {@link StatusLine} instance.
    * 
    * @return  instance of UI-specific implementation of {@link StatusLine}.
    */
   StatusLine createStatusLine();
}
